@@ -3,9 +3,9 @@
 // rustdoc-stripper-ignore-next
 //! Traits intended for implementing the [`CompletionProposal`](crate::CompletionProposal) interface.
 
-use crate::CompletionProposal;
+use crate::{prelude::*, CompletionProposal};
 use glib::subclass::prelude::*;
 
-pub trait CompletionProposalImpl: ObjectImpl {}
+pub trait CompletionProposalImpl: ObjectImpl + ObjectSubclass<Type: IsA<glib::Object>> {}
 
 unsafe impl<T: CompletionProposalImpl> IsImplementable<T> for CompletionProposal {}
