@@ -10,7 +10,7 @@ use glib::subclass::prelude::*;
 use glib::translate::*;
 use std::{future::Future, pin::Pin};
 
-pub trait CompletionProviderImpl: ObjectImpl {
+pub trait CompletionProviderImpl: ObjectImpl + ObjectSubclass<Type: IsA<glib::Object>> {
     fn activate(&self, context: &CompletionContext, proposal: &CompletionProposal) {
         self.parent_activate(context, proposal)
     }
@@ -59,7 +59,7 @@ pub trait CompletionProviderImpl: ObjectImpl {
     }
 
     fn populate(&self, context: &CompletionContext) -> Result<gio::ListModel, glib::Error> {
-        self.parnet_populate(context)
+        self.parent_populate(context)
     }
 
     fn populate_future(
@@ -70,30 +70,23 @@ pub trait CompletionProviderImpl: ObjectImpl {
     }
 }
 
-mod sealed {
-    pub trait Sealed {}
-    impl<T: super::CompletionProviderImplExt> Sealed for T {}
-}
-
-pub trait CompletionProviderImplExt: sealed::Sealed + ObjectSubclass {
+pub trait CompletionProviderImplExt: CompletionProviderImpl {
     fn parent_activate(&self, context: &CompletionContext, proposal: &CompletionProposal) {
         unsafe {
             let type_data = Self::type_data();
             let parent_iface = type_data.as_ref().parent_interface::<CompletionProvider>()
                 as *const ffi::GtkSourceCompletionProviderInterface;
 
-            let func = (*parent_iface)
-                .activate
-                .expect("no parent \"activate\" implementation");
-
-            func(
-                self.obj()
-                    .unsafe_cast_ref::<CompletionProvider>()
-                    .to_glib_none()
-                    .0,
-                context.to_glib_none().0,
-                proposal.to_glib_none().0,
-            )
+            if let Some(func) = (*parent_iface).activate {
+                func(
+                    self.obj()
+                        .unsafe_cast_ref::<CompletionProvider>()
+                        .to_glib_none()
+                        .0,
+                    context.to_glib_none().0,
+                    proposal.to_glib_none().0,
+                )
+            }
         }
     }
 
@@ -103,16 +96,16 @@ pub trait CompletionProviderImplExt: sealed::Sealed + ObjectSubclass {
             let parent_iface = type_data.as_ref().parent_interface::<CompletionProvider>()
                 as *const ffi::GtkSourceCompletionProviderInterface;
 
-            let func = (*parent_iface)
-                .get_title
-                .expect("no parent \"get_title\" implementation");
-
-            from_glib_full(func(
-                self.obj()
-                    .unsafe_cast_ref::<CompletionProvider>()
-                    .to_glib_none()
-                    .0,
-            ))
+            if let Some(func) = (*parent_iface).get_title {
+                from_glib_full(func(
+                    self.obj()
+                        .unsafe_cast_ref::<CompletionProvider>()
+                        .to_glib_none()
+                        .0,
+                ))
+            } else {
+                None
+            }
         }
     }
 
@@ -127,19 +120,17 @@ pub trait CompletionProviderImplExt: sealed::Sealed + ObjectSubclass {
             let parent_iface = type_data.as_ref().parent_interface::<CompletionProvider>()
                 as *const ffi::GtkSourceCompletionProviderInterface;
 
-            let func = (*parent_iface)
-                .display
-                .expect("no parent \"display\" implementation");
-
-            func(
-                self.obj()
-                    .unsafe_cast_ref::<CompletionProvider>()
-                    .to_glib_none()
-                    .0,
-                context.to_glib_none().0,
-                proposal.to_glib_none().0,
-                cell.to_glib_none().0,
-            )
+            if let Some(func) = (*parent_iface).display {
+                func(
+                    self.obj()
+                        .unsafe_cast_ref::<CompletionProvider>()
+                        .to_glib_none()
+                        .0,
+                    context.to_glib_none().0,
+                    proposal.to_glib_none().0,
+                    cell.to_glib_none().0,
+                )
+            }
         }
     }
 
@@ -149,17 +140,17 @@ pub trait CompletionProviderImplExt: sealed::Sealed + ObjectSubclass {
             let parent_iface = type_data.as_ref().parent_interface::<CompletionProvider>()
                 as *const ffi::GtkSourceCompletionProviderInterface;
 
-            let func = (*parent_iface)
-                .get_priority
-                .expect("no parent \"get_priority\" implementation");
-
-            func(
-                self.obj()
-                    .unsafe_cast_ref::<CompletionProvider>()
-                    .to_glib_none()
-                    .0,
-                context.to_glib_none().0,
-            )
+            if let Some(func) = (*parent_iface).get_priority {
+                func(
+                    self.obj()
+                        .unsafe_cast_ref::<CompletionProvider>()
+                        .to_glib_none()
+                        .0,
+                    context.to_glib_none().0,
+                )
+            } else {
+                0
+            }
         }
     }
 
@@ -169,18 +160,18 @@ pub trait CompletionProviderImplExt: sealed::Sealed + ObjectSubclass {
             let parent_iface = type_data.as_ref().parent_interface::<CompletionProvider>()
                 as *const ffi::GtkSourceCompletionProviderInterface;
 
-            let func = (*parent_iface)
-                .is_trigger
-                .expect("no parent \"is_trigger\" implementation");
-
-            from_glib(func(
-                self.obj()
-                    .unsafe_cast_ref::<CompletionProvider>()
-                    .to_glib_none()
-                    .0,
-                iter.to_glib_none().0,
-                c.into_glib(),
-            ))
+            if let Some(func) = (*parent_iface).is_trigger {
+                from_glib(func(
+                    self.obj()
+                        .unsafe_cast_ref::<CompletionProvider>()
+                        .to_glib_none()
+                        .0,
+                    iter.to_glib_none().0,
+                    c.into_glib(),
+                ))
+            } else {
+                false
+            }
         }
     }
 
@@ -194,19 +185,19 @@ pub trait CompletionProviderImplExt: sealed::Sealed + ObjectSubclass {
             let parent_iface = type_data.as_ref().parent_interface::<CompletionProvider>()
                 as *const ffi::GtkSourceCompletionProviderInterface;
 
-            let func = (*parent_iface)
-                .list_alternates
-                .expect("no parent \"list_alternates\" implementation");
-
-            let output = func(
-                self.obj()
-                    .unsafe_cast_ref::<CompletionProvider>()
-                    .to_glib_none()
-                    .0,
-                context.to_glib_none().0,
-                proposal.to_glib_none().0,
-            );
-            FromGlibPtrArrayContainerAsVec::from_glib_full_as_vec(output)
+            if let Some(func) = (*parent_iface).list_alternates {
+                let output = func(
+                    self.obj()
+                        .unsafe_cast_ref::<CompletionProvider>()
+                        .to_glib_none()
+                        .0,
+                    context.to_glib_none().0,
+                    proposal.to_glib_none().0,
+                );
+                FromGlibPtrArrayContainerAsVec::from_glib_full_as_vec(output)
+            } else {
+                vec![]
+            }
         }
     }
 
@@ -216,18 +207,16 @@ pub trait CompletionProviderImplExt: sealed::Sealed + ObjectSubclass {
             let parent_iface = type_data.as_ref().parent_interface::<CompletionProvider>()
                 as *const ffi::GtkSourceCompletionProviderInterface;
 
-            let func = (*parent_iface)
-                .refilter
-                .expect("no parent \"refilter\" implementation");
-
-            func(
-                self.obj()
-                    .unsafe_cast_ref::<CompletionProvider>()
-                    .to_glib_none()
-                    .0,
-                context.to_glib_none().0,
-                model.to_glib_none().0,
-            )
+            if let Some(func) = (*parent_iface).refilter {
+                func(
+                    self.obj()
+                        .unsafe_cast_ref::<CompletionProvider>()
+                        .to_glib_none()
+                        .0,
+                    context.to_glib_none().0,
+                    model.to_glib_none().0,
+                )
+            }
         }
     }
 
@@ -243,24 +232,24 @@ pub trait CompletionProviderImplExt: sealed::Sealed + ObjectSubclass {
             let parent_iface = type_data.as_ref().parent_interface::<CompletionProvider>()
                 as *const ffi::GtkSourceCompletionProviderInterface;
 
-            let func = (*parent_iface)
-                .key_activates
-                .expect("no parent \"key_activates\" implementation");
-
-            from_glib(func(
-                self.obj()
-                    .unsafe_cast_ref::<CompletionProvider>()
-                    .to_glib_none()
-                    .0,
-                context.to_glib_none().0,
-                proposal.to_glib_none().0,
-                keyval.into_glib(),
-                state.into_glib(),
-            ))
+            if let Some(func) = (*parent_iface).key_activates {
+                from_glib(func(
+                    self.obj()
+                        .unsafe_cast_ref::<CompletionProvider>()
+                        .to_glib_none()
+                        .0,
+                    context.to_glib_none().0,
+                    proposal.to_glib_none().0,
+                    keyval.into_glib(),
+                    state.into_glib(),
+                ))
+            } else {
+                false
+            }
         }
     }
 
-    fn parnet_populate(&self, context: &CompletionContext) -> Result<gio::ListModel, glib::Error> {
+    fn parent_populate(&self, context: &CompletionContext) -> Result<gio::ListModel, glib::Error> {
         unsafe {
             let type_data = Self::type_data();
             let parent_iface = type_data.as_ref().parent_interface::<CompletionProvider>()
