@@ -8,7 +8,7 @@ use glib::subclass::prelude::*;
 use glib::translate::*;
 use std::pin::Pin;
 
-pub trait HoverProviderImpl: ObjectImpl {
+pub trait HoverProviderImpl: ObjectImpl + ObjectSubclass<Type: IsA<glib::Object>> {
     fn populate_future(
         &self,
         context: &HoverContext,
@@ -18,12 +18,7 @@ pub trait HoverProviderImpl: ObjectImpl {
     }
 }
 
-mod sealed {
-    pub trait Sealed {}
-    impl<T: super::HoverProviderImplExt> Sealed for T {}
-}
-
-pub trait HoverProviderImplExt: sealed::Sealed + ObjectSubclass {
+pub trait HoverProviderImplExt: HoverProviderImpl {
     fn parent_populate_async<P: FnOnce(Result<(), glib::Error>) + 'static>(
         &self,
         context: &HoverContext,
